require 'open3'

require 'ygg/provisioner/model'

module Ygg
module Vhosts
module Provisioner

class Vhost < Ygg::Provisioner::Model

  self.state_persistant_keys = [:state]

  def provision!
    bender = config.vhosts_provisioner[:bender] || '/usr/bin/bender'
    loglevel = config.vhosts_provisioner[:bender_log_level] || 'info'
    if Process.uid > 0
      bender = "/usr/bin/sudo #{bender}"
    end
    run "#{bender} --provision webserver --#{loglevel} --service #{self.username}"
  end

  def deprovision!
    bender = config.vhosts_provisioner[:bender] || '/usr/bin/bender'
    loglevel = config.vhosts_provisioner[:bender_log_level] || 'info'
    if Process.uid > 0
      bender = "/usr/bin/sudo #{bender}"
    end
    run "#{bender} --deprovision webserver --#{loglevel} --service #{self.username}"
  end

  def provision_letsencrypt!
    bender = config.vhosts_provisioner[:bender] || '/usr/bin/bender'
    loglevel = config.vhosts_provisioner[:bender_log_level] || 'info'
    if Process.uid > 0
      bender = "/usr/bin/sudo #{bender}"
    end
    run "#{bender} --update letsencrypt --#{loglevel} --service #{self.username}"
  end

  protected

  def run(cmd, env = {})
    status = nil
    plog.debug "  + command: '#{cmd}'"
    begin
      Open3.popen3(env, "#{cmd}") do |stdin, stdout, stderr, thread|
        stdin.close_write
        eof = false
        outbuf = ''
        errbuf = ''
        begin
          files = [stdout, stderr]

          until eof do
            ready = IO.select(files)

            if ready
              readable = ready[0]

              readable.each do |f|
                next if f.eof
                if f == stdout
                  outbuf += f.read_nonblock(1024)
                  lines = outbuf.split(/(\r\n|\r|\n)/)
                  outbuf = lines.pop
                  lines.each do | l |
                    next if l.strip == ''
                    plog.info("-> #{l}")
                  end
                else
                  errbuf += f.read_nonblock(1024)
                  lines = errbuf.split(/(\r\n|\r|\n)/)
                  errbuf = lines.pop
                  lines.each do | l |
                    next if l.strip == ''
                    plog.error("-> #{l}")
                  end
                end
              end
            end
            eof = stdout.eof && stderr.eof
          end
          lines = outbuf.split(/(\r\n|\r|\n)/)
          lines.each do | l |
            next if l.strip == ''
            plog.info("-> #{l}")
          end
          lines = errbuf.split(/(\r\n|\r|\n)/)
          lines.each do | l |
            next if l.strip == ''
            plog.error("-> #{l}")
          end
        end
        status = thread.value
      end
      status.success?
    rescue SystemCallError => ex
      plog.error("-> #{ex}")
      false
    end
  end

end

end
end
end
